# Copyright (c) 2013, University of Bristol and University of Southampton.

from EStat.Templating import Template

class SPSSCrossTab(Template):
    'Create a cross-tabulation in SPSS (equivalent to Analyse -> Descriptive Statistics -> Crosstabs... in SPSS 23)'

    __version__ = '1.0.0'

    tags = ['SPSS', 'Crosstab']
    engines = ['SPSS_script']

    inputs = '''
rowvar = DataVector('Row variable: ')
colvar = DataVector('Column variable: ')
'''

    spssscript = '''
CROSSTABS 
  /TABLES=${rowvar} BY ${colvar}
  /CELLS=\\
  COUNT\\
  .

CROSSTABS 
  /TABLES=${rowvar} BY ${colvar}
  /CELLS=\\
  COUNT\\
  ROW\\
  .

CROSSTABS 
  /TABLES=${rowvar} BY ${colvar}
  /CELLS=\\
  COUNT\\
  COLUMN\\
  .

CROSSTABS 
  /TABLES=${rowvar} BY ${colvar}
  /CELLS=\\
  COUNT\\
  TOTAL\\
  .
'''


    processoutputs = '''
import EStat
from EStat.Templating import *

import numpy
import random
import base64

from lxml import etree
from io import BytesIO

ns = {'x': 'http://www.ibm.com/software/analytics/spss/xml/oms'}

def str2float(s):
    try:
        return float(s)
    except ValueError:
        return float('nan')

caseout =  etree.parse(BytesIO(outputs['Table_Case Processing Summary.xml'].content))
print(etree.tostring(caseout, pretty_print=True))

path = "string(//x:dimension[@axis='row']/x:category/@text)"
casetext = caseout.xpath(path, namespaces=ns)
print(casetext)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Valid']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nvalid = int(caseout.xpath(path, namespaces=ns))
print(Nvalid)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Missing']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Nmissing = int(caseout.xpath(path, namespaces=ns))
print(Nmissing)

path = "string(//x:dimension[@axis='row']/x:category/x:dimension[@text='Cases']/x:category[@text='Total']/x:dimension[@text='Statistics']/x:category[@text='N']/x:cell/@number)"
Ntotal = int(caseout.xpath(path, namespaces=ns))
print(Ntotal)

crosstabout = etree.parse(BytesIO(outputs['Table_Crosstabulation.xml'].content))
print(etree.tostring(crosstabout, pretty_print=True))

path = "string(//x:dimension[@axis='layer']/x:category[@text='Count']/x:dimension[@axis='row']/@label)"
rowlabel = crosstabout.xpath(path, namespaces=ns)
print(rowlabel)

path = "string(//x:dimension[@axis='layer']/x:category[@text='Count']/x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='column']/@label)"
collabel = crosstabout.xpath(path, namespaces=ns)
print(collabel)

path = "//x:dimension[@axis='row']/x:group/x:category/@label"
rowcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
if rowcats == []:
     path = "//x:dimension[@axis='row']/x:group/x:category/@text"
     rowcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
print(rowcats)


path = "//x:dimension[@axis='row']/x:category/x:dimension[@axis='column']/x:group/x:category/@label"
colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
if colcats == []:
    path = "//x:dimension[@axis='row']/x:category/x:dimension[@axis='column']/x:group/x:category/@text"
    colcats = [x for x in crosstabout.xpath(path, namespaces=ns)]
print(colcats)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
counts = numpy.array([int(x) for x in crosstabout.xpath(path, namespaces=ns)])
counts = counts.reshape([len(rowcats), len(colcats)])
print(counts)

path = "//x:dimension[@axis='row']/x:group/x:category/x:dimension[@axis='column']/x:category[@text='Total']/x:cell/@text"
countcoltots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
print(countcoltots)

path = "//x:dimension[@axis='row']/x:category[@text='Total']/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
countrowtots = [int(x) for x in crosstabout.xpath(path, namespaces=ns)]
print(countrowtots)

crosstabout1 = etree.parse(BytesIO(outputs['Table_Crosstabulation_1.xml'].content))
print(etree.tostring(crosstabout1, pretty_print=True))

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
percentrow_num = numpy.array([str2float(x) for x in crosstabout1.xpath(path, namespaces=ns)])
percentrow_num = percentrow_num.reshape([len(rowcats)+1, len(colcats)])
print(percentrow_num)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
percentrow_str = numpy.array(crosstabout1.xpath(path, namespaces=ns))
percentrow_str = percentrow_str.reshape([len(rowcats)+1, len(colcats)])
print(percentrow_str)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:category/x:cell/@number"
percentrowtots_num = [str2float(x) for x in crosstabout1.xpath(path, namespaces=ns)]
print(percentrowtots_num)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:category/x:cell/@text"
percentrowtots_str = crosstabout1.xpath(path, namespaces=ns)
print(percentrowtots_str)

crosstabout2 = etree.parse(BytesIO(outputs['Table_Crosstabulation_2.xml'].content))
print(etree.tostring(crosstabout2, pretty_print=True))

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
percentcol_num = numpy.array([str2float(x) for x in crosstabout2.xpath(path, namespaces=ns)])
percentcol_num = percentcol_num.reshape([len(rowcats)+1, len(colcats)])
print(percentcol_num)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
percentcol_str = numpy.array(crosstabout2.xpath(path, namespaces=ns))
percentcol_str = percentcol_str.reshape([len(rowcats)+1, len(colcats)])
print(percentcol_str)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:category/x:cell/@number"
percentcoltots_num = [str2float(x) for x in crosstabout2.xpath(path, namespaces=ns)]
print(percentcoltots_num)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:category/x:cell/@text"
percentcoltots_str = crosstabout2.xpath(path, namespaces=ns)
print(percentcoltots_str)

crosstabout3 = etree.parse(BytesIO(outputs['Table_Crosstabulation_3.xml'].content))
print(etree.tostring(crosstabout3, pretty_print=True))

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@number"
percenttot_num = numpy.array([str2float(x) for x in crosstabout3.xpath(path, namespaces=ns)])
percenttot_num = percenttot_num.reshape([len(rowcats)+1, len(colcats)])
print(percenttot_num)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:group/x:category/x:cell/@text"
percenttot_str = numpy.array(crosstabout3.xpath(path, namespaces=ns))
percenttot_str = percenttot_str.reshape([len(rowcats)+1, len(colcats)])
print(percenttot_str)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:category/x:cell/@number"
percenttottots_num = [str2float(x) for x in crosstabout3.xpath(path, namespaces=ns)]
print(percenttottots_num)

path = "//x:dimension[@axis='row']/x:category[contains(@text, '%')]/x:dimension[@axis='column']/x:category/x:cell/@text"
percenttottots_str = crosstabout3.xpath(path, namespaces=ns)
print(percenttottots_str)
   
varname1short = str(rowvar)
varname1long = str(rowvar)
if rowlabel != '':
    varname1long = rowlabel + '[' + varname1long + ']'

varname1short = '<strong>' + varname1short + '</strong>'
varname1long = '<strong>' + varname1long + '</strong>'

varname2short = str(colvar)
varname2long = str(colvar)
if collabel != '':
    varname2long = collabel + '[' + varname2long + ']'

varname2short = '<strong>' + varname2short + '</strong>'
varname2long = '<strong>' + varname2long + '</strong>'

rowfortext1 = random.randint(0, len(rowcats) - 1)
rowfortextcompare = rowfortext1
while rowfortextcompare == rowfortext1:
    rowfortextcompare = random.randint(0, len(rowcats) - 1)

colfortext1 = random.randint(0, len(colcats) - 1)
colfortextcompare = colfortext1
while colfortextcompare == colfortext1:
    colfortextcompare = random.randint(0, len(colcats) - 1)

rowfortext2 = random.randint(0, len(rowcats) - 1)
colfortext2 = random.randint(0, len(colcats) - 1)

rowfortext3 = random.randint(0, len(rowcats) - 1)
colfortext3 = random.randint(0, len(colcats) - 1)

rowfortext4 = random.randint(0, len(rowcats) - 1)
colfortext4 = random.randint(0, len(colcats) - 1)

introhtml = '<h2>Tabulation practical</h2>'
introhtml += '<p>'
introhtml += 'In this practical we will look at the basic cross-tabulation options in SPSS. '
introhtml += 'Cross tabulations are useful for summarising the relationships between pairs of categorical variables. '
introhtml += '</p>'
outputs['introduction.html'] = HTMLOutput(introhtml, "Introduction text")

counthtml = '<p>'
counthtml += 'We will begin by performing the simple cross-tabulation of the variables ' + varname1short + ' and ' + varname2short + ' using the SPSS instructions shown below:'
counthtml += '</p>'

countinstrhtml = '<ul class = "instructions">'
countinstrhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
countinstrhtml += '<li>Copy the <strong>' + varname1long + '</strong> variable into the <strong>Row(s)</strong> box.</li>'
countinstrhtml += '<li>Copy the <strong>' + varname2long +'</strong> variable into the <strong>Column(s)</strong> box.</li>'
countinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the table as shown below.</li>'
countinstrhtml += '</ul>'

counthtml += countinstrhtml

counthtml += outputs['Table_Crosstabulation.htm'].content
counthtml += '<p>'
counthtml += 'This simple cross tabulation simply looks at the variables in the full dataset and looks at how many times each combination of our two categorical variables appears. '
counthtml += 'So we can see that there are ' + str(counts[rowfortext1, colfortext1]) + ' occurrences where <strong>' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext1]) + ' and <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + '. '
counthtml += 'This is out of '+  str(countcoltots[rowfortext1]) + ' occurences in total where <strong>' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext1]) + ' and '+ str(countrowtots[colfortext1]) + ' occurrences in total where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + '.'
counthtml += '</p>'

counthtml += '<p>'
counthtml += 'From this table we can see that when ' + varname1short + ' takes value ' + str(rowcats[rowfortext1]) + ' '
if counts[rowfortext1, colfortext1] > counts[rowfortext1, colfortextcompare]:
    counthtml += 'there are more instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[rowfortextcompare]) + '. '
    if countrowtots[colfortext1] > countrowtots[colfortextcompare]:
        counthtml += 'This may be because overall there are more instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
    else:
        counthtml += 'This is surprising as overall there are not more instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
elif counts[rowfortext1, colfortextcompare] > counts[rowfortext1, colfortext1]:
    counthtml += 'there are fewer instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
    if countrowtots[colfortext1] < countrowtots[colfortextcompare]:
        counthtml += 'This may be because overall there are fewer instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
    else: 
        counthtml += 'This is surprising as overall there are not fewer instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
else: 
    counthtml += 'there are equal numbers of instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' and ' + str(colcats[colfortextcompare]) + '. '
counthtml += '</p>'

outputs['counts.html'] = HTMLOutput(counthtml, "Counts text")

countquesthtml = '<p>'
countquesthtml += 'Use the Crosstabs window  to construct a cross-tabulation in SPSS to answer the following questions: '
countquesthtml += '</p>'
countquesthtml += '<ul>'
countquesthtml += '<li>Question: How many times does <strong>' + varname1short + '</strong> take value ' + str(rowcats[rowfortext1]) + ' when <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + '?</li>'
countquesthtml += '<li>Question: When <strong>' + varname1short + '</strong> take value ' + str(rowcats[rowfortext1]) + ' are there more values where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + ' or where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortextcompare]) + '?</li>'
countquesthtml += '<li>Question: Overall does<strong> '+ varname2short + '</strong> take value ' + str(colcats[colfortext1]) +' or value ' + str(colcats[colfortextcompare]) + ' more frequently?</li>'
countquesthtml += '</ul>'
outputs['countquestions.html'] = HTMLOutput(countquesthtml, "Counts question text")

countsolhtml = '<p>'
countsolhtml += 'The SPSS instructions for the basic cross-tab are as follows: '
countsolhtml += '</p>'
countsolhtml += '<ul class = "instructions">'
countsolhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
countsolhtml += '<li>Copy the <strong>' + varname1long + '</strong> variable into the <strong>Row(s)</strong> box.</li>'
countsolhtml += '<li>Copy the <strong>' + varname2long +'</strong> variable into the <strong>Column(s)</strong> box.</li>'
countsolhtml += '<li>Click on the <strong>OK</strong> button to produce the required table.</li>'
countsolhtml += '</ul>'
countsolhtml += '<ul>'
countsolhtml += '<li>Question: How many times does <strong>' + varname1short + '</strong> take value ' + str(rowcats[rowfortext1]) + ' when <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + '?</li>'
countsolhtml += '<li>Question: When <strong>' + varname1short + '</strong> take value ' + str(rowcats[rowfortext1]) + ' are there more values where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + ' or where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortextcompare]) + '?</li>'
countsolhtml += '<li>Question: Overall does<strong> ' + varname2short + '</strong> take value ' + str(colcats[colfortext1]) +' or value ' + str(colcats[colfortextcompare]) + ' more frequently?</li>'
countsolhtml += '</ul>'
countsolhtml += '<p>'
countsolhtml += 'Solutions: The output from SPSS is as follows:'
countsolhtml += '</p>'
countsolhtml += outputs['Table_Crosstabulation.htm'].content
countsolhtml += '<p>'
countsolhtml += 'The answers are as follows:'
countsolhtml += '</p>'
countsolhtml += '<ul>'
countsolhtml += '<li>There are ' + str(counts[rowfortext1, colfortext1]) + ' occurences where <strong>' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext1]) + ' and <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext1]) + '.</li> '
countsolhtml += '<li>From this table we can see that when ' + varname1short + ' takes value ' + str(rowcats[rowfortext1]) + ' '
if counts[rowfortext1, colfortext1] > counts[rowfortext1, colfortextcompare]:
    countsolhtml += 'there are more instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[rowfortextcompare]) + '. '
elif counts[rowfortext1, colfortextcompare] > counts[rowfortext1, colfortext1]:
    countsolhtml += 'there are fewer instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
else: 
    countsolhtml += 'there are equal numbers of instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' and ' + str(colcats[colfortextcompare]) + '. '
countsolhtml += '</li>'
countsolhtml += '<li>'
if countrowtots[colfortext1] > countrowtots[colfortextcompare]:
    countsolhtml += 'Overall there are more instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
elif countrowtots[colfortext1] < countrowtots[colfortextcompare]:
    countsolhtml += 'Overall there are more instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' than value ' + str(colcats[colfortextcompare]) + '. '
else:
    countsolhtml += 'Overall there are equal numbers of instances of ' + varname2short + ' taking value '+ str(colcats[colfortext1]) + ' and ' + str(colcats[colfortextcompare]) + '. '
countsolhtml += '</li>'
countsolhtml += '</ul>'
outputs['countanswers.html'] = HTMLOutput(countsolhtml, "Counts answer text")

centrowinstrhtml = '<p>'
centrowinstrhtml += 'It can be helpful in understanding the individual cell counts to look at how their relative frequencies within a particular row or column compare to overall relative frequencies for all rows and columns. '
centrowinstrhtml += 'To do this we look at percentage values as well as counts and we will do this first for rows by following the instructions below:'
centrowinstrhtml += '</p>'

centrowinstrhtml += '<ul class = "instructions">'
centrowinstrhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
centrowinstrhtml += '<li>The Row and Column variables should be already chosen so Click on the <strong>Cells...</strong> button.</li>'
centrowinstrhtml += '<li>Click on the <strong>Row</strong> tickbox found under the <strong>Percentages</strong> section.</li>'
centrowinstrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
centrowinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the table as shown below.</li>'
centrowinstrhtml += '</ul>'

centrowhtml = centrowinstrhtml

centrowhtml += outputs['Table_Crosstabulation_1.htm'].content

centrowhtml += '<p>'
centrowhtml += 'This new table contains the same information as the last table but with the addition of percentages underneath each count. '
centrowhtml += 'These percentages represent what percentage of the observations in each row are found in each cell. '
centrowhtml += 'So we see that when ' + varname1short + ' takes value ' + str(rowcats[rowfortext2]) + ' then for ' + varname2short + ' ' 
for loop in range(0, len(colcats)):
    if loop == len(colcats) - 1:
        centrowhtml += ' and '
    elif loop > 0:
        centrowhtml += ', '
    centrowhtml += percentrow_str[rowfortext2, loop] + ' (or ' + str(counts[rowfortext2, loop]) + ') take value ' + str(colcats[loop])
centrowhtml += '. '
centrowhtml += '</p>'

centrowhtml += '<p>'
centrowhtml += 'This compares with the overall distribution of ' + varname2short + ' where '
for loop in range(0, len(colcats)):
    if loop == len(colcats) - 1:
        centrowhtml += ' and '
    elif loop > 0:
        centrowhtml += ', '
    centrowhtml += percentrow_str[-1, loop] + ' (or ' + str(countrowtots[loop]) + ') take value ' + str(colcats[loop])
centrowhtml += '. '

centrowhtml += 'So for example we see that when ' + varname1short + ' takes value ' + str(rowcats[rowfortext2]) + ' there is '
if percentrow_num[rowfortext2, colfortext2] > percentrow_num[-1, colfortext2]:
    centrowhtml += 'a greater percentage ('  + percentrow_str[rowfortext2, colfortext2] + ') '
elif percentrow_num[rowfortext2, colfortext2] < percentrow_num[-1, colfortext2]:
    centrowhtml += 'a smaller percentage ('  + percentrow_str[rowfortext2, colfortext2] + ') '
else:
    centrowhtml += 'the same percentage ('  + percentrow_str[rowfortext2, colfortext2] + ') '
centrowhtml += 'taking category ' + str(colcats[colfortext2]) + ' than on average (' + percentrow_str[-1, colfortext2] + '). '
centrowhtml += '</p>'
outputs['percentrow.html'] = HTMLOutput(centrowhtml, "Row percentage text")

centrowquesthtml = '<p>'
centrowquesthtml += 'Use the Crosstabs window with row percentages in SPSS to answer the following questions: '
centrowquesthtml += '</p>'
centrowquesthtml += '<ul>'
centrowquesthtml += '<li>Question: What percentage of observations where <strong>' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext2]) + ' has <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext2]) + '?</li>'
centrowquesthtml += '<li>Question: What percentage of all observations has <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext2]) + '?</li>'
centrowquesthtml += '<li>Question: How does the percentage of observations with <strong> '+ varname2short + '</strong> taking value ' + str(colcats[colfortext2]) +' when ' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext2]) + ' compare to the overall distribution?</li>'
centrowquesthtml += '</ul>'
outputs['percentrowquestions.html'] = HTMLOutput(centrowquesthtml, "Row percentage question text")

centrowsolhtml = '<p>'
centrowsolhtml += 'The SPSS instructions for the cross-tab with row percentages are as follows: '
centrowsolhtml += '</p>'
centrowsolhtml += '<ul class = "instructions">'
centrowsolhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
centrowsolhtml += '<li>The Row and Column variables should be already chosen so Click on the <strong>Cells...</strong> button.</li>'
centrowsolhtml += '<li>Click on the <strong>Row</strong> tickbox found under the <strong>Percentages</strong> section.</li>'
centrowsolhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
centrowsolhtml += '<li>Click on the <strong>OK</strong> button to produce the required table.</li>'
centrowsolhtml += '</ul>'
centrowsolhtml += '<ul>'
centrowsolhtml += '<li>Question: What percentage of observations where <strong>' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext2]) + ' has <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext2]) + '?</li>'
centrowsolhtml += '<li>Question: What percentage of all observations has <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext2]) + '?</li>'
centrowsolhtml += '<li>Question: How does the percentage of observation with <strong> '+ varname2short + '</strong> taking value ' + str(colcats[colfortext2]) +' when ' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext2]) + ' compare to the overall distribution?</li>'
centrowsolhtml += '</ul>'
centrowsolhtml += '<p>'
centrowsolhtml += 'Solutions: The output from SPSS is as follows:'
centrowsolhtml += '</p>'
centrowsolhtml += outputs['Table_Crosstabulation_1.htm'].content
centrowsolhtml += '<p>'
centrowsolhtml += 'The answers are as follows:'
centrowsolhtml += '</p>'
centrowsolhtml += '<ul>'
centrowsolhtml += '<li>' + percentrow_str[rowfortext2, colfortext2] + ' of observations have ' + varname2short + '</strong> taking value ' + str(colcats[colfortext2]) + ' when <strong>' + varname1short + '</strong> takes value ' + str(rowcats[rowfortext2]) + '.</li> '
centrowsolhtml += '<li>' + percentrow_str[-1, colfortext2] + ' of all observations have <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext2]) + '</li>'
centrowsolhtml += '<li>'
if percentrow_num[rowfortext2, colfortext2] > percentrow_num[-1, colfortext2]:
    centrowsolhtml += 'a greater percentage of observations ('  + percentrow_str[rowfortext2, colfortext2] + ') '
elif percentrow_num[rowfortext2, colfortext2] < percentrow_num[-1, colfortext2]:
    centrowsolhtml += 'a smaller percentage of observations ('  + percentrow_str[rowfortext2, colfortext2] + ') '
else:
    centrowsolhtml += 'the same percentage of observations ('  + percentrow_str[rowfortext2, colfortext2] + ') '
centrowsolhtml += 'take category ' + str(colcats[colfortext2]) + ' for ' + varname2short + ' when ' + varname1short + ' takes value ' + str(rowcats[rowfortext2]) + ' than on average (' + percentrow_str[-1, colfortext2] + '). '
centrowsolhtml += '</li>'
centrowsolhtml += '</ul>'
outputs['percentrowanswers.html'] = HTMLOutput(centrowsolhtml, "Row percentage answer text")

centcolinstrhtml = '<p>'
centcolinstrhtml += 'We can now look at columns instead of rows via the following:'
centcolinstrhtml += '</p>'

centcolinstrhtml += '<ul class = "instructions">'
centcolinstrhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
centcolinstrhtml += '<li>The Row and Column variables should be already chosen so Click on the <strong>Cells...</strong> button.</li>'
centcolinstrhtml += '<li>Remove the <strong>Row</strong> tickbox and instead click on the <strong>Column</strong> tickbox under the <strong>Percentages</strong> section.</li>'
centcolinstrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
centcolinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the table as shown below.</li>'
centcolinstrhtml += '</ul>'

centcolhtml = centcolinstrhtml

centcolhtml += outputs['Table_Crosstabulation_2.htm'].content

centcolhtml += '<p>'
centcolhtml += 'This new table contains the same information as the last table but this time the percentages underneath each count are column rather than row percentages. '
centcolhtml += 'These percentages represent what percentage of the observations in each column are found in each cell. '
centcolhtml += 'So we see that when ' + varname2short + ' takes value ' + str(colcats[colfortext3]) + ' then for ' + varname1short + ' ' 
for loop in range(0, len(rowcats)):
    if loop == len(rowcats) - 1:
        centcolhtml += ' and '
    elif loop > 0:
        centcolhtml += ', '
    centcolhtml += percentcol_str[loop, colfortext3] + ' (or ' + str(counts[loop, colfortext3]) + ') take value ' + str(rowcats[loop])
centcolhtml += '. '
centcolhtml += '</p>'

centcolhtml += '<p>'
centcolhtml += 'This compares with the overall distribution of ' + varname1short + ' where '
for loop in range(0, len(rowcats)):
    if loop == len(rowcats) - 1:
        centcolhtml += ' and '
    elif loop > 0:
        centcolhtml += ', '
    centcolhtml += percentcoltots_str[loop] + ' (or ' + str(countcoltots[loop]) + ') take value ' + str(rowcats[loop])
centcolhtml += '. '

centcolhtml += 'So for example we see that when ' + varname2short + ' takes value ' + str(colcats[colfortext3]) + ' there is '
if percentcol_num[rowfortext3, colfortext3] > percentcoltots_num[colfortext3]:
    centcolhtml += 'a greater percentage ('  + percentcol_str[rowfortext3, colfortext3] + ') '
elif percentcol_num[rowfortext3, colfortext3] < percentcoltots_num[colfortext3]:
    centcolhtml += 'a smaller percentage ('  + percentcol_str[rowfortext3, colfortext3] + ') '
else:
    centcolhtml += 'the same percentage ('  + percentcol_str[rowfortext3, colfortext3] + ') '
centcolhtml += 'taking category ' + str(rowcats[rowfortext3]) + ' than on average (' + percentcoltots_str[rowfortext3] + '). '
centcolhtml += '</p>'

outputs['percentcolumn.html'] = HTMLOutput(centcolhtml, "Column percentage text")

centcolquesthtml = '<p>'
centcolquesthtml += 'Use the Crosstabs window with column percentages in SPSS to answer the following questions: '
centcolquesthtml += '</p>'
centcolquesthtml += '<ul>'
centcolquesthtml += '<li>Question: What percentage of observations where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext3]) + ' has <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) + '?</li>'
centcolquesthtml += '<li>Question: What percentage of all observations has <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) + '?</li>'
centcolquesthtml += '<li>Question: How does the percentage of observations with <strong> '+ varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) +' when ' + varname2short + '</strong> takes value ' + str(colcats[colfortext3]) + ' compare to the overall distribution?</li>'
centcolquesthtml += '</ul>'
outputs['percentcolumnquestions.html'] = HTMLOutput(centcolquesthtml, "Column percentage question text")

centcolsolhtml = '<p>'
centcolsolhtml += 'The SPSS instructions for the cross-tab with column percentages are as follows: '
centcolsolhtml += '</p>'
centcolsolhtml += '<ul class = "instructions">'
centcolsolhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
centcolsolhtml += '<li>The Row and Column variables should be already chosen so Click on the <strong>Cells...</strong> button.</li>'
centcolsolhtml += '<li>Remove the <strong>Row</strong> tickbox and instead click on the <strong>Column</strong> tickbox under the <strong>Percentages</strong> section.</li>'
centcolsolhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
centcolsolhtml += '<li>Click on the <strong>OK</strong> button to produce the required table.</li>'
centcolsolhtml += '</ul>'
centcolsolhtml += '<ul>'
centcolsolhtml += '<li>Question: What percentage of observations where <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext3]) + ' has <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) + '?</li>'
centcolsolhtml += '<li>Question: What percentage of all observations has <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) + '?</li>'
centcolsolhtml += '<li>Question: How does the percentage of observations with <strong> '+ varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) +' when ' + varname2short + '</strong> takes value ' + str(colcats[colfortext3]) + ' compare to the overall distribution?</li>'
centcolsolhtml += '</ul>'
centcolsolhtml += '<p>'
centcolsolhtml += 'Solutions: The output from SPSS is as follows:'
centcolsolhtml += '</p>'
centcolsolhtml += outputs['Table_Crosstabulation_2.htm'].content
centcolsolhtml += '<p>'
centcolsolhtml += 'The answers are as follows:'
centcolsolhtml += '</p>'
centcolsolhtml += '<ul>'
centcolsolhtml += '<li>' + percentcol_str[rowfortext3, colfortext3] + ' of observations have ' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) + ' when <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext3]) + '.</li> '
centcolsolhtml += '<li>' + percentcoltots_str[rowfortext3] + ' of all observations have <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext3]) + '</li>'
centcolsolhtml += '<li>'
if percentcol_num[rowfortext3, colfortext3] > percentcoltots_num[rowfortext3]:
    centcolsolhtml += 'a greater percentage of observations ('  + percentrow_str[rowfortext3, colfortext3] + ') '
elif percentcol_num[rowfortext3, colfortext3] < percentcoltots_num[rowfortext3]:
    centcolsolhtml += 'a smaller percentage of observations ('  + percentrow_str[rowfortext3, colfortext3] + ') '
else:
    centcolsolhtml += 'the same percentage of observations ('  + percentrow_str[rowfortext3, colfortext3] + ') '
centcolsolhtml += 'take category ' + str(rowcats[rowfortext3]) + ' for ' + varname1short + ' when ' + varname2short + ' takes value ' + str(colcats[colfortext3]) + ' than on average (' + percentcoltots_str[rowfortext3] + '). '
centcolsolhtml += '</li>'
centcolsolhtml += '</ul>'
outputs['percentcolumnanswers.html'] = HTMLOutput(centcolsolhtml, "Column percentage answer text")

centtotinstrhtml = '<p>'
centtotinstrhtml += 'Finally for completeness we can look at total percentages as follows:'
centtotinstrhtml += '</p>'

centtotinstrhtml += '<ul class = "instructions">'
centtotinstrhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
centtotinstrhtml += '<li>The Row and Column variables should be already chosen so Click on the <strong>Cells...</strong> button.</li>'
centtotinstrhtml += '<li>Remove the <strong>Column</strong> tickbox and instead click on the <strong>Total</strong> tickbox under the <strong>Percentages</strong> section.</li>'
centtotinstrhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
centtotinstrhtml += '<li>Click on the <strong>OK</strong> button to produce the table as shown below.</li>'
centtotinstrhtml += '</ul>'

centtothtml = centtotinstrhtml

centtothtml += outputs['Table_Crosstabulation_3.htm'].content

centtothtml += '<p>'
centtothtml += 'In this final table the percentages represent percentages out of all the observations. '
centtothtml += 'So for example the ' + str(counts[rowfortext4, colfortext4]) + ' observations where ' + varname1short + ' takes value ' + str(rowcats[rowfortext4]) + ' and ' + varname2short + ' takes value ' + str(colcats[colfortext4]) + ' represent ' + percenttot_str[rowfortext4, colfortext4] + ' of all the observations. '
centtothtml += '</p>'
outputs['percenttotal.html'] = HTMLOutput(centtothtml, "Total percentage text")

centtotquesthtml = '<p>'
centtotquesthtml += 'Use the Crosstabs window with total percentages in SPSS to answer the following questions: '
centtotquesthtml += '</p>'
centtotquesthtml += '<ul>'
centtotquesthtml += '<li>Question: What percentage of all observation have <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext4]) + ' and <strong>' + varname2short + '</strong> takes value ' + str(colcats[colfortext4]) + '?</li>'
centtotquesthtml += '</ul>'
outputs['percenttotalquestions.html'] = HTMLOutput(centtotquesthtml, "Total percentage question text")

centtotsolhtml = '<p>'
centtotsolhtml += 'The SPSS instructions for the cross-tab with cell percentages are as follows: '
centtotsolhtml += '</p>'
centtotsolhtml += '<ul class = "instructions">'
centtotsolhtml += '<li>Select <strong>Crosstabs...</strong> from the <strong>Descriptive Statistics</strong> submenu available from the <strong>Analyze</strong> menu.</li>'
centtotsolhtml += '<li>The Row and Column variables should be already chosen so Click on the <strong>Cells...</strong> button.</li>'
centtotsolhtml += '<li>Remove the <strong>Column</strong> tickbox and instead click on the <strong>Total</strong> tickbox under the <strong>Percentages</strong> section.</li>'
centtotsolhtml += '<li>Click on the <strong>Continue</strong> button to return to the main window.</li>'
centtotsolhtml += '<li>Click on the <strong>OK</strong> button to produce the table as shown below.</li>'
centtotsolhtml += '</ul>'
centtotsolhtml += '<ul>'
centtotsolhtml += '<li>Question: What percentage of all observations have <strong>' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext4]) + ' and <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext4]) + '?</li>'
centtotsolhtml += '</ul>'
centtotsolhtml += '<p>'
centtotsolhtml += 'Solutions: The output from SPSS is as follows:'
centtotsolhtml += '</p>'
centtotsolhtml += outputs['Table_Crosstabulation_3.htm'].content
centtotsolhtml += '<p>'
centtotsolhtml += 'The answers are as follows:'
centtotsolhtml += '</p>'
centtotsolhtml += '<ul>'
centtotsolhtml += '<li>' + percenttot_str[rowfortext4, colfortext4] + ' of all observations have ' + varname1short + '</strong> taking value ' + str(rowcats[rowfortext4]) + ' and <strong>' + varname2short + '</strong> taking value ' + str(colcats[colfortext4]) + '.</li> '
centtotsolhtml += '</ul>'
outputs['percenttotalanswers.html'] = HTMLOutput(centtotsolhtml, "Total percentage answer text")

conchtml = '<p>'
conchtml += 'These percentages are probably the least useful of the three options as they do not really aid in comparisons within the table but might aid comparisons between different tables. '
conchtml += 'This ends the practical on tabulations. '
conchtml += '</p>'
outputs['conclusion.html'] = HTMLOutput(conchtml, "Conclusion text")

combinedprachtml = outputs['introduction.html'].content
combinedprachtml += outputs['counts.html'].content
combinedprachtml += outputs['percentrow.html'].content
combinedprachtml += outputs['percentcolumn.html'].content
combinedprachtml += outputs['percenttotal.html'].content
combinedprachtml += outputs['conclusion.html'].content

outputs['allpractical.html'] = HTMLOutput(combinedprachtml, "Combined practical HTML outputs")

combinedquesthtml = outputs['introduction.html'].content
combinedquesthtml += outputs['countquestions.html'].content
combinedquesthtml += outputs['percentrowquestions.html'].content
combinedquesthtml += outputs['percentcolumnquestions.html'].content
combinedquesthtml += outputs['percenttotalquestions.html'].content

outputs['allquestions.html'] = HTMLOutput(combinedquesthtml, "Combined questions HTML outputs")

combinedsolhtml = outputs['countanswers.html'].content
combinedsolhtml += outputs['percentrowanswers.html'].content
combinedsolhtml += outputs['percentcolumnanswers.html'].content
combinedsolhtml += outputs['percenttotalanswers.html'].content

outputs['allanswers.html'] = HTMLOutput(combinedsolhtml, "Combined solutions HTML outputs")
'''
   

